<?php
/**
* Plugin Name: CQPIM Project Management
* Description: CQPIM is a solution for freelancers and small agencies who want to manage their clients, quotes, projects and invoices more efficiently. Work individually or as part of a team, and start streamlining your processes!
* Version: 2.9.6
* Author: TimelessInteractive+
* Author URI: http://www.cqpim.uk/docs
* Text Domain: cqpim
* Domain Path: /languages
*/
?>

<?php
	update_option('cqpim_29', true);

	// Theme Compatibilities

	function cqpim_theme_compat() {

		$theme = wp_get_theme();
		
		if($theme->name == 'X') {
		
			$dash_page = get_option('cqpim_client_page');
			
			if(is_page($dash_page) || is_singular('cqpim_project') || is_singular('cqpim_quote') || is_singular('cqpim_support') || is_singular('cqpim_invoice')) {
			
				wp_dequeue_script('x-site-head');
			
			}
		
		}
	
	}
	
	add_action( 'wp_print_scripts', 'cqpim_theme_compat', 100 );

	$dashboard = get_option('client_dashboard_type');
	
	if(empty($dashboard)) {
	
		update_option('client_dashboard_type', 'inc');
	
	}

	// Languages
	
	function cqpim_plugin_setup() {
	
		if(is_admin()) {
	
			load_plugin_textdomain('cqpim', false, dirname(plugin_basename(__FILE__)) . '/languages/admin/');
			
		} else {
		
			load_plugin_textdomain('cqpim', false, dirname(plugin_basename(__FILE__)) . '/languages/frontend/');
		
		}
		
	} 
	
	add_action('plugins_loaded', 'cqpim_plugin_setup');

	// Include Required Files
 
	// Main functions
 
	require_once( 'cqpim_functions.php' );
	
	require_once( 'cqpim_admin.php' );
	
	// Admin Dashboard & Settings
	
	require_once( 'plugin-settings.php' );
	
	require_once( 'dashboard.php' );
	
		require_once( 'noerror.php' );
		
		require_once( 'calendar.php' );
		
		require_once( 'tasks.php' );
		
		require_once( 'tasks_admin.php' );
		
		require_once( 'support-tickets.php' );
		
		require_once( 'capabilities.php' );
		
		
		
		// CPT's
		
		require_once( 'cpt/teams.php' );
		
		require_once( 'cpt/clients.php' );
		
		if(get_option('enable_quotes') == 1) {
		
			require_once( 'cpt/quotes.php' );
		
			require_once( 'cpt/forms.php' );
		
		}
		
		require_once( 'cpt/projects.php' );
		
		require_once( 'cpt/terms.php' );
		
		if(get_option('disable_invoices') != 1) {
		
			require_once( 'cpt/invoices.php' );
		
		}
		
		require_once( 'cpt/tasks.php' );
		
		require_once( 'cpt/support.php' );
		
		require_once( 'cpt/templates.php' );
		
		
		// Metaboxes
		
		require_once( 'metaboxes/index.php' );
		
		// Shortcodes
		
		require_once( 'shortcodes/shortcodes.php' );
	
	// Script Enqueing
	
	require_once( 'script-enqueing.php' );
	
	// Add custom event scheduler
	
	function cqpim_schedule_minute( $schedules ) {
	 
		$schedules['every_minute'] = array(
		
				'interval'  => 60,
				
				'display'   => __( 'Every Minute', 'cqpim' )
				
		);
		 
		return $schedules;
		
	}
	
	add_filter( 'cron_schedules', 'cqpim_schedule_minute' );
	
	
	// Plugin Activation	
	
	register_activation_hook( __FILE__, 'cqpim_activation' );
	
	function cqpim_activation() {
	
		// Set up hourly checking of recurring invoices
		
		wp_schedule_event(time(), 'hourly', 'cqpim_check_recurring_invoices');
		
		// set up minute checking of email piping
		
		wp_schedule_event(time(), 'every_minute', 'cqpim_check_email_pipe');
	
		// Add custom client role 
		
		$role = get_role('cqpim_client');
		
		if(empty($role)) {
		 
			$result = add_role( 'cqpim_client', __( 'CQPIM Client' ),
			 
				array(
			 
					'read' => true,
					'read_cqpim_quotes' => true,
					'read_cqpim_projects' => true,
					'read_cqpim_supports' => true,
					'read_private_cqpim_quotes' => true,
					'read_private_cqpim_projects' => true,
					'read_private_cqpim_supports' => true,
			 
				)
			 
			);
			
			$result = add_role( 'cqpim_admin', __( 'CQPIM Admin' ),
			 
				array(
			 
					'read' => true,
					
					'upload_files' => true,
			 
				)
			 
			);
			
			$cqpim_roles = array('cqpim_admin', 'user');
			
			update_option('cqpim_roles', $cqpim_roles);
		
		}
		
		
	
	// Create the CQPIM Client Login Page

		$new_page_title = 'CQPIM Login';
		
		$new_page_content = '';
		
		$new_page_template = '';

		$page_check = get_page_by_title($new_page_title);
		
		$new_page = array(
		
				'post_type' => 'page',
				
				'post_title' => $new_page_title,
				
				'post_content' => $new_page_content,
				
				'post_excerpt' => '',
				
				'post_status' => 'publish',
				
				'post_author' => 1,
				
				'post_name' => 'cqpim-login',
				
				'comment_status' => 'closed'
				
		);
		
		if(!isset($page_check->ID)){
		
				$new_page_id = wp_insert_post($new_page);
				
				update_option( 'cqpim_login_page', $new_page_id, true );
				
				if(!empty($new_page_template)){
				
						update_post_meta($new_page_id, '_wp_page_template', $new_page_template);
						
				}
				
		}

	// Create the CQPIM Client Dashboard Page

		$new_page_title = 'CQPIM Client Dashboard';
		
		$new_page_content = '';
		
		$new_page_template = '';

		$page_check = get_page_by_title($new_page_title);
		
		$new_page = array(
		
				'post_type' => 'page',
				
				'post_title' => $new_page_title,
				
				'post_content' => $new_page_content,
				
				'post_status' => 'publish',
				
				'post_author' => 1,
				
				'post_name' => 'cqpim-client',
				
				'post_excerpt' => '',
				
				'comment_status' => 'closed'
				
		);
		
		if(!isset($page_check->ID)){
		
				$new_page_id = wp_insert_post($new_page);
				
				update_option( 'cqpim_client_page', $new_page_id, true );
				
				if(!empty($new_page_template)){
				
						update_post_meta($new_page_id, '_wp_page_template', $new_page_template);
						
				}
				
		}
		
			$new_page_title = 'CQPIM Password Reset';
			
			$new_page_content = '';
			
			$new_page_template = '';

			$page_check = get_page_by_title($new_page_title);
			
			$new_page = array(
			
					'post_type' => 'page',
					
					'post_title' => $new_page_title,
					
					'post_content' => $new_page_content,
					
					'post_excerpt' => '',
					
					'post_status' => 'publish',
					
					'post_author' => 1,
					
					'post_name' => 'cqpim-reset',
					
					'comment_status' => 'closed'
					
			);
			
			if(!isset($page_check->ID)){
			
					$new_page_id = wp_insert_post($new_page);
					
					update_option( 'cqpim_reset_page', $new_page_id, true );
					
					if(!empty($new_page_template)){
					
							update_post_meta($new_page_id, '_wp_page_template', $new_page_template);
							
					}
					
			}
		
		update_option('client_dashboard_type', 'inc');
		
		//cqpim_set_options();
	
	}
	
	// Plugin deactivation
	
	register_deactivation_hook( __FILE__, 'cqpim_deactivation' );
	
	function cqpim_deactivation() {
	
		//remove_role('cqpim_client');
		
		//remove_role('cqpim_admin');
		
		$client_login = get_option('cqpim_login_page');
		
		$client_dash = get_option('cqpim_client_page');
		
		$client_reset = get_option('cqpim_reset_page');
		
		wp_delete_post($client_login, true);
		
		wp_delete_post($client_reset, true);
		
		wp_delete_post($client_dash, true);
		
		// Create custom roles add in plugin

		$roles = get_option('cqpim_roles'); 

		if(!is_array($roles)) {

			$roles = array(get_option('cqpim_roles'));

		}

		foreach($roles as $role) {

			$cqpim_role = 'cqpim_' . $role;

			remove_role($cqpim_role);

		}
		
		//delete_option('cqpim_roles');
		
		//delete_option('cqpim_permissions');
	
	}
	
	// Manage Roles
	
	add_action('admin_init', 'cqpim_manage_roles');
	
	function cqpim_manage_roles() {
	
		$roles = get_option('cqpim_roles'); 
		
		if(!is_array($roles)) {
		
			$roles = array(get_option('cqpim_roles'));
		
		}
		
		if($roles) {
		
			foreach($roles as $role) {
			
				if($role != 'cqpim_admin') {
			
					$cqpim_role = 'cqpim_' . $role;
				
					$role_exists = get_role($cqpim_role);
					
					if(!$role_exists) {
					
						$caps = array(
						
							'read' => true,
						
						);
						
						$role_name = str_replace('_', ' ', $cqpim_role);
						
						$role_name = str_replace('cqpim', 'CQPIM', $role_name);
						
						$role_name = ucwords($role_name);
						
						add_role($cqpim_role, $role_name, $caps);
					
					}
				
				}
			
			}
		
		}

		if ( !function_exists('get_editable_roles') ) {
		
		require_once( ABSPATH . '/wp-admin/includes/user.php' );
		
		}
		
		$system_roles = get_editable_roles();

		$plugin_roles = get_option('cqpim_roles');

		if(!is_array($plugin_roles)) {

			$plugin_roles = array(get_option('cqpim_roles'));

		}

		$plugin_roles_ready = array();
		
		foreach($plugin_roles as $plugin_role) {
		
			if($plugin_role == 'cqpim_admin') {
			
				$plugin_roles_ready[] = $plugin_role;
				
			} else {
		
				$plugin_roles_ready[] = 'cqpim_' . $plugin_role;
			
			}
		
		}	

		foreach($system_roles as $key => $system_role) {
		
			if(strpos($key,'cqpim_') !== false && !in_array($key, $plugin_roles_ready)) {
			
				if($key != 'cqpim_client') {
			
					remove_role($key);
				
				}
			
			}
		
		}	
	
	}
	
	// Admin Capabilities (CQPIM Admin and Administrator get everything!)
	
    add_action('admin_init','cqpim_add_role_caps',10);
	
    function cqpim_add_role_caps() {

		// Add the roles you'd like to administer the custom post types
		$roles = array('cqpim_admin', 'administrator');
		
		// Loop through each role and assign capabilities
		foreach($roles as $the_role) { 

		    $role = get_role($the_role);
			
			// Custom CQPIM Permissions
			
			$role->add_cap('cqpim_view_dashboard');
			$role->add_cap('cqpim_view_tickets');
			$role->add_cap('edit_cqpim_permissions');
			$role->add_cap('cqpim_create_new_client');
			$role->add_cap('cqpim_create_new_form');
			$role->add_cap('cqpim_reset_client_passwords');
			$role->add_cap('cqpim_create_new_team');
			$role->add_cap('cqpim_reset_team_passwords');
			$role->add_cap('edit_cqpim_help');
			$role->add_cap('edit_cqpim_settings');
			$role->add_cap('cqpim_grant_admin_role');
			$role->add_cap('cqpim_create_new_quote');
			$role->add_cap('cqpim_send_quotes');
			$role->add_cap('cqpim_view_project_financials');
			$role->add_cap('cqpim_view_all_projects');
			$role->add_cap('cqpim_publish_all_projects');
			$role->add_cap('cqpim_send_project_messages');
			$role->add_cap('cqpim_edit_project_milestones');
			$role->add_cap('cqpim_edit_project_members');
			$role->add_cap('cqpim_edit_project_dates');
			$role->add_cap('cqpim_dash_view_all_tasks');
			$role->add_cap('cqpim_create_new_invoice');			
			$role->add_cap('cqpim_send_invoices');			
			$role->add_cap('cqpim_mark_invoice_paid');			
			$role->add_cap('cqpim_view_project_contract');
			$role->add_cap('cqpim_view_project_client_page');
			$role->add_cap('cqpim_mark_project_signedoff');
			$role->add_cap('cqpim_mark_project_closed');
			$role->add_cap('cqpim_create_new_project');
			$role->add_cap('cqpim_create_new_terms');
			$role->add_cap('cqpim_create_new_templates');
			$role->add_cap('cqpim_apply_project_templates');
			$role->add_cap('cqpim_create_new_supports');
			
			$role->add_cap('edit_posts');
			$role->add_cap('edit_private_posts');			
			$role->add_cap('cqpim_do_all');						
			$role->add_cap('upload_files');	
			
			// CPT Permissions
			
	        $role->add_cap( 'read_cqpim_client');
	        $role->add_cap( 'read_private_cqpim_clients' );
	        $role->add_cap( 'edit_cqpim_clients' );
	        $role->add_cap( 'edit_others_cqpim_clients' );
	        $role->add_cap( 'edit_published_cqpim_clients' );
			$role->add_cap( 'edit_private_cqpim_clients' );
	        $role->add_cap( 'publish_cqpim_clients' );
			$role->add_cap( 'delete_cqpim_clients' );
	        $role->add_cap( 'delete_others_cqpim_clients' );
	        $role->add_cap( 'delete_private_cqpim_clients' );
	        $role->add_cap( 'delete_published_cqpim_clients' );
			
	        $role->add_cap( 'read_cqpim_task');
	        $role->add_cap( 'read_private_cqpim_tasks' );
	        $role->add_cap( 'edit_cqpim_tasks' );
	        $role->add_cap( 'edit_others_cqpim_tasks' );
	        $role->add_cap( 'edit_published_cqpim_tasks' );
			$role->add_cap( 'edit_private_cqpim_tasks' );
	        $role->add_cap( 'publish_cqpim_tasks' );
			
	        $role->add_cap( 'read_cqpim_template');
	        $role->add_cap( 'read_private_cqpim_templates' );
	        $role->add_cap( 'edit_cqpim_templates' );
	        $role->add_cap( 'edit_others_cqpim_templates' );
	        $role->add_cap( 'edit_published_cqpim_templates' );
			$role->add_cap( 'edit_private_cqpim_templates' );
	        $role->add_cap( 'publish_cqpim_templates' );
			$role->add_cap( 'delete_cqpim_templates' );
	        $role->add_cap( 'delete_others_cqpim_templates' );
	        $role->add_cap( 'delete_private_cqpim_templates' );
	        $role->add_cap( 'delete_published_cqpim_templates' );
			
	        $role->add_cap( 'read_cqpim_term');
	        $role->add_cap( 'read_private_cqpim_terms' );
	        $role->add_cap( 'edit_cqpim_terms' );
	        $role->add_cap( 'edit_others_cqpim_terms' );
	        $role->add_cap( 'edit_published_cqpim_terms' );
			$role->add_cap( 'edit_private_cqpim_terms' );
	        $role->add_cap( 'publish_cqpim_terms' );
			$role->add_cap( 'delete_cqpim_terms' );
	        $role->add_cap( 'delete_others_cqpim_terms' );
	        $role->add_cap( 'delete_private_cqpim_terms' );
	        $role->add_cap( 'delete_published_cqpim_terms' );
			
	        $role->add_cap( 'read_cqpim_form');
	        $role->add_cap( 'read_private_cqpim_forms' );
	        $role->add_cap( 'edit_cqpim_forms' );
	        $role->add_cap( 'edit_others_cqpim_forms' );
	        $role->add_cap( 'edit_published_cqpim_forms' );
			$role->add_cap( 'edit_private_cqpim_forms' );
	        $role->add_cap( 'publish_cqpim_forms' );
			$role->add_cap( 'delete_cqpim_forms' );
	        $role->add_cap( 'delete_others_cqpim_forms' );
	        $role->add_cap( 'delete_private_cqpim_forms' );
	        $role->add_cap( 'delete_published_cqpim_forms' );
			
	        $role->add_cap( 'read_cqpim_team');
	        $role->add_cap( 'read_private_cqpim_teams' );
	        $role->add_cap( 'edit_cqpim_teams' );
	        $role->add_cap( 'edit_others_cqpim_teams' );
	        $role->add_cap( 'edit_published_cqpim_teams' );
			$role->add_cap( 'edit_private_cqpim_teams' );
	        $role->add_cap( 'publish_cqpim_teams' );
			$role->add_cap( 'delete_cqpim_teams' );
	        $role->add_cap( 'delete_others_cqpim_teams' );
	        $role->add_cap( 'delete_private_cqpim_teams' );
	        $role->add_cap( 'delete_published_cqpim_teams' );
			
	        $role->add_cap( 'read_cqpim_support');
	        $role->add_cap( 'read_private_cqpim_supports' );
	        $role->add_cap( 'edit_cqpim_supports' );
	        $role->add_cap( 'edit_others_cqpim_supports' );
	        $role->add_cap( 'edit_published_cqpim_supports' );
			$role->add_cap( 'edit_private_cqpim_supports' );
	        $role->add_cap( 'publish_cqpim_supports' );
			$role->add_cap( 'delete_cqpim_supports' );
	        $role->add_cap( 'delete_others_cqpim_supports' );
	        $role->add_cap( 'delete_private_cqpim_supports' );
	        $role->add_cap( 'delete_published_cqpim_supports' );
			
	        $role->add_cap( 'read_cqpim_project');
	        $role->add_cap( 'read_private_cqpim_projects' );
	        $role->add_cap( 'edit_cqpim_projects' );
	        $role->add_cap( 'edit_others_cqpim_projects' );
	        $role->add_cap( 'edit_published_cqpim_projects' );
			$role->add_cap( 'edit_private_cqpim_projects' );
	        $role->add_cap( 'publish_cqpim_projects' );
			$role->add_cap( 'delete_cqpim_projects' );
	        $role->add_cap( 'delete_others_cqpim_projects' );
	        $role->add_cap( 'delete_private_cqpim_projects' );
	        $role->add_cap( 'delete_published_cqpim_projects' );
			
	        $role->add_cap( 'read_cqpim_quote');
	        $role->add_cap( 'read_private_cqpim_quotes' );
	        $role->add_cap( 'edit_cqpim_quotes' );
	        $role->add_cap( 'edit_others_cqpim_quotes' );
	        $role->add_cap( 'edit_published_cqpim_quotes' );
			$role->add_cap( 'edit_private_cqpim_quotes' );
	        $role->add_cap( 'publish_cqpim_quotes' );
			$role->add_cap( 'delete_cqpim_quotes' );
	        $role->add_cap( 'delete_others_cqpim_quotes' );
	        $role->add_cap( 'delete_private_cqpim_quotes' );
	        $role->add_cap( 'delete_published_cqpim_quotes' );
			
	        $role->add_cap( 'read_cqpim_invoice');
	        $role->add_cap( 'read_private_cqpim_invoices' );
	        $role->add_cap( 'edit_cqpim_invoices' );
	        $role->add_cap( 'edit_others_cqpim_invoices' );
	        $role->add_cap( 'edit_published_cqpim_invoices' );
			$role->add_cap( 'edit_private_cqpim_invoices' );
	        $role->add_cap( 'publish_cqpim_invoices' );
			$role->add_cap( 'delete_cqpim_invoices' );
	        $role->add_cap( 'delete_others_cqpim_invoices' );
	        $role->add_cap( 'delete_private_cqpim_invoices' );
	        $role->add_cap( 'delete_published_cqpim_invoices' );		
		}
		
	}
	
	// Custom Role Caps
	
	add_action('admin_init','cqpim_custom_role_caps');
	
	function cqpim_custom_role_caps() {
	
		$cqpim_roles = get_option('cqpim_roles');
		
		$cqpim_perms = get_option('cqpim_permissions');
		
		if(!is_array($cqpim_roles)) {
		
			$cqpim_roles = array(get_option('cqpim_roles'));
		
		}	

		$roles_to_assign = array();
		
		foreach($cqpim_roles as $cqpim_role) {
		
			if($cqpim_role != 'cqpim_admin') {
			
				$roles_to_assign[] = 'cqpim_' . $cqpim_role;
			
			}
		
		}
		
		foreach($roles_to_assign as $the_role) {
		
			$role = get_role($the_role);
			
			$role->add_cap('edit_posts');
			$role->add_cap('edit_private_posts');
			$role->add_cap('cqpim_view_dashboard');	
			
			// Grant Read/Update on all Projects that are assigned
			
	        $role->add_cap( 'read_cqpim_project');
	        $role->add_cap( 'read_private_cqpim_projects' );
	        $role->add_cap( 'edit_cqpim_projects' );
	        $role->add_cap( 'edit_others_cqpim_projects' );
	        $role->add_cap( 'edit_published_cqpim_projects' );
			$role->add_cap( 'edit_private_cqpim_projects' );
	        $role->add_cap( 'publish_cqpim_projects' );
			
	        $role->add_cap( 'read_cqpim_task');
	        $role->add_cap( 'read_private_cqpim_tasks' );
	        $role->add_cap( 'edit_cqpim_tasks' );
	        $role->add_cap( 'edit_others_cqpim_tasks' );
	        $role->add_cap( 'edit_published_cqpim_tasks' );
			$role->add_cap( 'edit_private_cqpim_tasks' );
	        $role->add_cap( 'publish_cqpim_tasks' );

			$role_to_check = str_replace('cqpim_', '', $the_role);
			
			if($cqpim_perms) {
			
			foreach($cqpim_perms as $key => $perm) {
			
				if(in_array($role_to_check, $perm)) {
				
					$role->add_cap($key);
					
					if($key == 'cqpim_view_tickets') {
					
						$role->add_cap( 'read_cqpim_support');
						$role->add_cap( 'read_private_cqpim_supports' );
						$role->add_cap( 'edit_cqpim_supports' );
						$role->add_cap( 'edit_others_cqpim_supports' );
						$role->add_cap( 'edit_published_cqpim_supports' );
						$role->add_cap( 'edit_private_cqpim_supports' );						
					
					}
					
					if($key == 'delete_cqpim_supports') {
					
						$role->add_cap( 'delete_others_cqpim_supports' );
						$role->add_cap( 'delete_private_cqpim_supports' );
						$role->add_cap( 'delete_published_cqpim_supports' );						
					
					}
					
					if($key == 'read_cqpim_client') {
					
						$role->add_cap( 'read_private_cqpim_clients' );
						$role->add_cap( 'edit_cqpim_clients' );
						$role->add_cap( 'edit_others_cqpim_clients' );
						$role->add_cap( 'edit_published_cqpim_clients' );
						$role->add_cap( 'edit_private_cqpim_clients' );					
					
					}
					
					if($key == 'delete_cqpim_clients') {
					
						$role->add_cap( 'delete_others_cqpim_clients' );
						$role->add_cap( 'delete_private_cqpim_clients' );
						$role->add_cap( 'delete_published_cqpim_clients' );						
					
					}
					
					if($key == 'read_cqpim_templates') {
					
						$role->add_cap( 'read_private_cqpim_templates' );
						$role->add_cap( 'edit_cqpim_templates' );
						$role->add_cap( 'edit_others_cqpim_templates' );
						$role->add_cap( 'edit_published_cqpim_templates' );
						$role->add_cap( 'edit_private_cqpim_templates' );					
					
					}
					
					if($key == 'delete_cqpim_templates') {
					
						$role->add_cap( 'delete_others_cqpim_templates' );
						$role->add_cap( 'delete_private_cqpim_templates' );
						$role->add_cap( 'delete_published_cqpim_templates' );						
					
					}
					
					if($key == 'read_cqpim_terms') {
					
						$role->add_cap( 'read_private_cqpim_terms' );
						$role->add_cap( 'edit_cqpim_terms' );
						$role->add_cap( 'edit_others_cqpim_terms' );
						$role->add_cap( 'edit_published_cqpim_terms' );
						$role->add_cap( 'edit_private_cqpim_terms' );					
					
					}
					
					if($key == 'delete_cqpim_terms') {
					
						$role->add_cap( 'delete_others_cqpim_terms' );
						$role->add_cap( 'delete_private_cqpim_terms' );
						$role->add_cap( 'delete_published_cqpim_terms' );						
					
					}
					
					if($key == 'read_cqpim_team') {
					
						$role->add_cap( 'read_private_cqpim_teams' );
						$role->add_cap( 'edit_cqpim_teams' );
						$role->add_cap( 'edit_others_cqpim_teams' );
						$role->add_cap( 'edit_published_cqpim_teams' );
						$role->add_cap( 'edit_private_cqpim_teams' );					
					
					}
					
					if($key == 'delete_cqpim_teams') {
					
						$role->add_cap( 'delete_others_cqpim_teams' );
						$role->add_cap( 'delete_private_cqpim_teams' );
						$role->add_cap( 'delete_published_cqpim_teams' );						
					
					}
					
					if($key == 'read_cqpim_quote') {
					
						$role->add_cap( 'read_private_cqpim_quotes' );
						$role->add_cap( 'edit_cqpim_quotes' );
						$role->add_cap( 'edit_others_cqpim_quotes' );
						$role->add_cap( 'edit_published_cqpim_quotes' );
						$role->add_cap( 'edit_private_cqpim_quotes' );					
					
					}
					
					if($key == 'delete_cqpim_quotes') {
					
						$role->add_cap( 'delete_others_cqpim_quotes' );
						$role->add_cap( 'delete_private_cqpim_quotes' );
						$role->add_cap( 'delete_published_cqpim_quotes' );						
					
					}
					
					if($key == 'read_cqpim_form') {
					
						$role->add_cap( 'read_private_cqpim_forms' );
						$role->add_cap( 'edit_cqpim_forms' );
						$role->add_cap( 'edit_others_cqpim_forms' );
						$role->add_cap( 'edit_published_cqpim_forms' );
						$role->add_cap( 'edit_private_cqpim_forms' );					
					
					}
					
					if($key == 'delete_cqpim_forms') {
					
						$role->add_cap( 'delete_others_cqpim_forms' );
						$role->add_cap( 'delete_private_cqpim_forms' );
						$role->add_cap( 'delete_published_cqpim_forms' );						
					
					}
					
					if($key == 'delete_cqpim_projects') {
					
						$role->add_cap( 'delete_others_cqpim_projects' );
						$role->add_cap( 'delete_private_cqpim_projects' );
						$role->add_cap( 'delete_published_cqpim_projects' );						
					
					}
					
					if($key == 'read_cqpim_invoice') {
					
						$role->add_cap( 'read_private_cqpim_invoices' );
						$role->add_cap( 'edit_cqpim_invoices' );
						$role->add_cap( 'edit_others_cqpim_invoices' );
						$role->add_cap( 'edit_published_cqpim_invoices' );
						$role->add_cap( 'edit_private_cqpim_invoices' );					
					
					}
					
					if($key == 'delete_cqpim_invoices') {
					
						$role->add_cap( 'delete_others_cqpim_invoices' );
						$role->add_cap( 'delete_private_cqpim_invoices' );
						$role->add_cap( 'delete_published_cqpim_invoices' );						
					
					}
				
				} else {
				
					$role->remove_cap($key);
					
					if($key == 'cqpim_view_tickets') {
					
						$role->remove_cap( 'read_cqpim_support');
						$role->remove_cap( 'read_private_cqpim_supports' );
						$role->remove_cap( 'edit_cqpim_supports' );
						$role->remove_cap( 'edit_others_cqpim_supports' );
						$role->remove_cap( 'edit_published_cqpim_supports' );
						$role->remove_cap( 'edit_private_cqpim_supports' );						
					
					}
					
					if($key == 'delete_cqpim_supports') {
					
						$role->remove_cap( 'delete_others_cqpim_supports' );
						$role->remove_cap( 'delete_private_cqpim_supports' );
						$role->remove_cap( 'delete_published_cqpim_supports' );						
					
					}
					
					if($key == 'read_cqpim_client') {
					
						$role->remove_cap( 'read_private_cqpim_clients' );
						$role->remove_cap( 'edit_cqpim_clients' );
						$role->remove_cap( 'edit_others_cqpim_clients' );
						$role->remove_cap( 'edit_published_cqpim_clients' );
						$role->remove_cap( 'edit_private_cqpim_clients' );					
					
					}
					
					if($key == 'delete_cqpim_clients') {
					
						$role->remove_cap( 'delete_others_cqpim_clients' );
						$role->remove_cap( 'delete_private_cqpim_clients' );
						$role->remove_cap( 'delete_published_cqpim_clients' );						
					
					}
					
					if($key == 'read_cqpim_templates') {
					
						$role->remove_cap( 'read_private_cqpim_templates' );
						$role->remove_cap( 'edit_cqpim_templates' );
						$role->remove_cap( 'edit_others_cqpim_templates' );
						$role->remove_cap( 'edit_published_cqpim_templates' );
						$role->remove_cap( 'edit_private_cqpim_templates' );					
					
					}
					
					if($key == 'delete_cqpim_templates') {
					
						$role->remove_cap( 'delete_others_cqpim_templates' );
						$role->remove_cap( 'delete_private_cqpim_templates' );
						$role->remove_cap( 'delete_published_cqpim_templates' );						
					
					}
					
					if($key == 'read_cqpim_terms') {
					
						$role->remove_cap( 'read_private_cqpim_terms' );
						$role->remove_cap( 'edit_cqpim_terms' );
						$role->remove_cap( 'edit_others_cqpim_terms' );
						$role->remove_cap( 'edit_published_cqpim_terms' );
						$role->remove_cap( 'edit_private_cqpim_terms' );					
					
					}
					
					if($key == 'delete_cqpim_terms') {
					
						$role->remove_cap( 'delete_others_cqpim_terms' );
						$role->remove_cap( 'delete_private_cqpim_terms' );
						$role->remove_cap( 'delete_published_cqpim_terms' );						
					
					}
					
					if($key == 'read_cqpim_team') {
					
						$role->remove_cap( 'read_private_cqpim_teams' );
						$role->remove_cap( 'edit_cqpim_teams' );
						$role->remove_cap( 'edit_others_cqpim_teams' );
						$role->remove_cap( 'edit_published_cqpim_teams' );
						$role->remove_cap( 'edit_private_cqpim_teams' );					
					
					}
					
					if($key == 'delete_cqpim_teams') {
					
						$role->remove_cap( 'delete_others_cqpim_teams' );
						$role->remove_cap( 'delete_private_cqpim_teams' );
						$role->remove_cap( 'delete_published_cqpim_teams' );						
					
					}
					
					if($key == 'read_cqpim_quote') {
					
						$role->remove_cap( 'read_private_cqpim_quotes' );
						$role->remove_cap( 'edit_cqpim_quotes' );
						$role->remove_cap( 'edit_others_cqpim_quotes' );
						$role->remove_cap( 'edit_published_cqpim_quotes' );
						$role->remove_cap( 'edit_private_cqpim_quotes' );					
					
					}
					
					if($key == 'delete_cqpim_quotes') {
					
						$role->remove_cap( 'delete_others_cqpim_quotes' );
						$role->remove_cap( 'delete_private_cqpim_quotes' );
						$role->remove_cap( 'delete_published_cqpim_quotes' );						
					
					}
					
					if($key == 'read_cqpim_form') {
					
						$role->remove_cap( 'read_private_cqpim_forms' );
						$role->remove_cap( 'edit_cqpim_forms' );
						$role->remove_cap( 'edit_others_cqpim_forms' );
						$role->remove_cap( 'edit_published_cqpim_forms' );
						$role->remove_cap( 'edit_private_cqpim_forms' );					
					
					}
					
					if($key == 'delete_cqpim_forms') {
					
						$role->remove_cap( 'delete_others_cqpim_forms' );
						$role->remove_cap( 'delete_private_cqpim_forms' );
						$role->remove_cap( 'delete_published_cqpim_forms' );						
					
					}
					
					if($key == 'delete_cqpim_projects') {
					
						$role->remove_cap( 'delete_others_cqpim_projects' );
						$role->remove_cap( 'delete_private_cqpim_projects' );
						$role->remove_cap( 'delete_published_cqpim_projects' );						
					
					}
					
					if($key == 'read_cqpim_invoice') {
					
						$role->remove_cap( 'read_private_cqpim_invoices' );
						$role->remove_cap( 'edit_cqpim_invoices' );
						$role->remove_cap( 'edit_others_cqpim_invoices' );
						$role->remove_cap( 'edit_published_cqpim_invoices' );
						$role->remove_cap( 'edit_private_cqpim_invoices' );					
					
					}
					
					if($key == 'delete_cqpim_invoices') {
					
						$role->remove_cap( 'delete_others_cqpim_invoices' );
						$role->remove_cap( 'delete_private_cqpim_invoices' );
						$role->remove_cap( 'delete_published_cqpim_invoices' );						
					
					}
				
				}
			
			}
			
			}
		
		}
	
	}
	
	
	// 2.6 Compat
	
	add_action('admin_init', 'cqpim_twopointsix_compat');
	
	function cqpim_twopointsix_compat() {
	
		$complete = get_option('twopoint6_completed');
	
		if($complete != '1') {
		
			$args = array(
			
				'post_type' => 'cqpim_client',
				
				'posts_per_page' => -1,
				
				'post_status' => 'private'
				
			);
			
			$clients = get_posts($args);
			
			foreach($clients as $client) {
			
				$client_details = get_post_meta($client->ID, 'client_details', true);
				
				$user_id = $client_details['user_id'];
				
				$client_ids = get_post_meta($client->ID, 'client_ids', true);
				
				if(empty($client_ids)) {
				
					$client_ids = array();
					
				}
				
				if(!in_array($user_id, $client_ids)) {
				
					$client_ids[] = $user_id;
				
				}
				
				update_post_meta($client->ID, 'client_ids', $client_ids);
			
			}
			
			// Move terms from settings to post
			
			$terms = get_option('default_contract_text');
			
			$user = wp_get_current_user();
			
			$new_terms = array(
			
				'post_type' => 'cqpim_terms',
				
				'post_status' => 'private',
				
				'post_content' => '',
				
				'post_author' => $user->ID,
				
				'post_title' => __('Default', 'cqpim'),
			
			);
			
			$terms_pid = wp_insert_post( $new_terms, false );

			update_post_meta($terms_pid, 'terms', $terms);
			
			update_option('default_contract_text', $terms_pid);
			
			update_option('twopoint6_completed', '1');
		
		}
	
	}
	
	// 2.7 Compat
	
	add_action('admin_init', 'cqpim_twopointseven_compat');
	
	function cqpim_twopointseven_compat() {
	
		$complete = get_option('twopoint7_completed');
		
		if($complete != '1') {
		
			update_option('cqpim_quote_slug', 'cqpim_quote');
			
			update_option('cqpim_invoice_slug', 'cqpim_invoice');
			
			update_option('cqpim_project_slug', 'cqpim_project');
			
			update_option('cqpim_support_slug', 'cqpim_support');
			
			update_option('cqpim_task_slug', 'cqpim_tasks');
		
			delete_option( 'client_update_email' );
			
			delete_option( 'client_update_subject' );
			
			delete_option( 'company_update_ticket_email' );
			
			delete_option( 'company_update_ticket_subject' );
		
			update_option('team_assignment_subject', 'Project %%TASK_PROJECT%%: Task Updated - %%TASK_TITLE%%.');
			
			update_option('cqpim_date_format', 'd/m/Y');
			
			$content = 'Dear %%NAME%%

%%CURRENT_USER%% has just updated a task: (%%TASK_TITLE%%)
				
Task Status: %%TASK_STATUS%%
Task Priority: %%TASK_PRIORITY%%
Start Date: %%TASK_START%%
Deadline: %%TASK_DEADLINE%%
Estimated Time (Hours): %%TASK_EST%%
Percentage Complete: %%TASK_PC%%

Project: %%TASK_PROJECT%%
Milestone: %%TASK_MILESTONE%%

Assignee: %%TASK_OWNER%%

%%TASK_UPDATE%%

Click here to see the updates -

%%TASK_URL%% 

Best Regards

%%CURRENT_USER%%

%%COMPANY_NAME%%
Tel: %%COMPANY_TELEPHONE%%
Email: %%COMPANY_SALES_EMAIL%%	
			
			';
			
			update_option('team_assignment_email', $content);
			
$content = 'Dear %%NAME%%

%%CURRENT_USER%% has just updated Support Ticket %%TICKET_ID%%

Title: %%TICKET_TITLE%%
ID: %%TICKET_ID%%
Priority: %%TICKET_PRIORITY%%
Status: %%TICKET_STATUS%%
Assignee: %%TICKET_OWNER%%

%%TICKET_UPDATE%%

Please log in to your dashboard to read and reply to their update.

Best Regards

%%COMPANY_NAME%% Support
Tel: %%COMPANY_TELEPHONE%%
Email: %%COMPANY_SALES_EMAIL%%
';			
		update_option('client_update_ticket_email', $content);
		
		update_option('client_update_ticket_subject', '%%CURRENT_USER%% has just updated Support Ticket %%TICKET_ID%%');
			
			// Replace Dates
			
			$args = array(
			
				'post_type' => 'cqpim_quote',
				
				'posts_per_page' => -1,
				
				'post_status' => 'private'
			
			);
			
			$quotes = get_posts($args);
			
			foreach($quotes as $quote) {
			
				$quote_details = get_post_meta($quote->ID, 'quote_details', true);
				
				if(!empty($quote_details['start_date']) && !is_numeric($quote_details['start_date'])) {
				
					$quote_details['start_date'] = cqpim_convert_date($quote_details['start_date']);
				
				}
				
				if(!empty($quote_details['finish_date']) && !is_numeric($quote_details['finish_date'])) {
				
					$quote_details['finish_date'] = cqpim_convert_date($quote_details['finish_date']);
				
				}
				
				if(!empty($quote_details['sent_details']['date']) && !is_numeric($quote_details['sent_details']['date'])) {
				
					$quote_details['sent_details']['date'] = cqpim_convert_date($quote_details['sent_details']['date'], 'd/m/Y H:i:s');
				
				}
				
				if(!empty($quote_details['confirmed_details']['date']) && !is_numeric($quote_details['confirmed_details']['date'])) {
				
					$quote_details['confirmed_details']['date'] = cqpim_convert_date($quote_details['confirmed_details']['date'], 'd/m/Y H:i:s');
				
				}
				
				update_post_meta($quote->ID, 'quote_details', $quote_details);
				
				$quote_elements = get_post_meta($quote->ID, 'quote_elements', true);
				
				if(!empty($quote_elements)) {
				
					foreach($quote_elements as $key => $element) {
					
						if(!empty($element['deadline']) && !is_numeric($element['deadline'])) {
						
							$quote_elements[$key]['deadline'] = cqpim_convert_date($element['deadline']);
						
						}

						if(!empty($element['start']) && !is_numeric($element['start'])) {
						
							$quote_elements[$key]['start'] = cqpim_convert_date($element['start']);
						
						}							
					
					}
				
				}
				
				update_post_meta($quote->ID, 'quote_elements', $quote_elements);
			
			}
			

			
			// Replace Dates
			
			$args = array(
			
				'post_type' => 'cqpim_project',
				
				'posts_per_page' => -1,
				
				'post_status' => 'private'
			
			);
			
			$quotes = get_posts($args);
			
			foreach($quotes as $quote) {
			
				$quote_details = get_post_meta($quote->ID, 'project_details', true);
				
				if(!empty($quote_details['start_date']) && !is_numeric($quote_details['start_date'])) {
				
					$quote_details['start_date'] = cqpim_convert_date($quote_details['start_date']);
				
				}
				
				if(!empty($quote_details['finish_date']) && !is_numeric($quote_details['finish_date'])) {
				
					$quote_details['finish_date'] = cqpim_convert_date($quote_details['finish_date']);
				
				}
				
				if(!empty($quote_details['sent_details']['date']) && !is_numeric($quote_details['sent_details']['date'])) {
				
					$quote_details['sent_details']['date'] = cqpim_convert_date($quote_details['sent_details']['date'], 'd/m/Y H:i:s');
				
				}
				
				if(!empty($quote_details['confirmed_details']['date']) && !is_numeric($quote_details['confirmed_details']['date'])) {
				
					$quote_details['confirmed_details']['date'] = cqpim_convert_date($quote_details['confirmed_details']['date'], 'd/m/Y H:i:s');
				
				}
				
				if(!empty($quote_details['signoff_details']['at']) && !is_numeric($quote_details['signoff_details']['at'])) {
				
					$quote_details['signoff_details']['at'] = cqpim_convert_date($quote_details['signoff_details']['at'], 'd/m/Y H:i:s');
				
				}
				
				if(!empty($quote_details['closed_details']['at']) && !is_numeric($quote_details['closed_details']['at'])) {
				
					$quote_details['closed_details']['at'] = cqpim_convert_date($quote_details['closed_details']['at'], 'd/m/Y H:i:s');
				
				}
				
				update_post_meta($quote->ID, 'project_details', $quote_details);
				
				$quote_elements = get_post_meta($quote->ID, 'project_elements', true);
				
				if(!empty($quote_elements)) {
				
					foreach($quote_elements as $key => $element) {
					
						if(!empty($element['deadline']) && !is_numeric($element['deadline'])) {
						
							$quote_elements[$key]['deadline'] = cqpim_convert_date($element['deadline']);
						
						}

						if(!empty($element['start']) && !is_numeric($element['start'])) {
						
							$quote_elements[$key]['start'] = cqpim_convert_date($element['start']);
						
						}							
					
					}
				
				}
				
				update_post_meta($quote->ID, 'project_elements', $quote_elements);
				
				$quote_elements = get_post_meta($quote->ID, 'project_progress', true);
				
				if(!empty($quote_elements)) {
				
					foreach($quote_elements as $key => $element) {
					
						if(!empty($element['date']) && !is_numeric($element['date'])) {
						
							$quote_elements[$key]['date'] = cqpim_convert_date($element['date'], 'd/m/Y H:i:s');
						
						}							
					
					}
				
				}
				
				update_post_meta($quote->ID, 'project_progress', $quote_elements);
			
			}
			
			$args = array(
			
				'post_type' => 'cqpim_invoice',
				
				'posts_per_page' => -1,
				
				'post_status' => 'publish'
				
			);
			
			$invoices = get_posts($args);
			
			foreach($invoices as $invoice) {
			
				$invoice_details = get_post_meta($invoice->ID, 'invoice_details', true);
				
				if(!empty($invoice_details['invoice_date']) && !is_numeric($invoice_details['invoice_date'])) {
				
					$invoice_details['invoice_date'] = preg_replace('/\s+/', '', $invoice_details['invoice_date']);
				
					$invoice_details['invoice_date'] = cqpim_convert_date($invoice_details['invoice_date']);
				
				}
				
				if(!empty($invoice_details['due']) && !is_numeric($invoice_details['due'])) {
				
					$invoice_details['due'] = cqpim_convert_date($invoice_details['due']);
				
				}
				
				if(!empty($invoice_details['sent_details']['date']) && !is_numeric($invoice_details['sent_details']['date'])) {
				
					$invoice_details['sent_details']['date'] = cqpim_convert_date($invoice_details['sent_details']['date'], 'd/m/Y H:i:s');
				
				}
				
				if(!empty($invoice_details['paid_details']['date']) && !is_numeric($invoice_details['paid_details']['date'])) {
				
					$invoice_details['paid_details']['date'] = cqpim_convert_date($invoice_details['paid_details']['date'], 'd/m/Y H:i:s');
				
				}
				
				update_post_meta($invoice->ID, 'invoice_details', $invoice_details);
			
			}
			
			$args = array(
			
				'post_type' => 'cqpim_support',
				
				'posts_per_page' => -1,
				
				'post_status' => 'private'
				
			);
			
			$invoices = get_posts($args);
			
			foreach($invoices as $invoice) {	

				$ticket_updated = get_post_meta($invoice->ID, 'last_updated', true);
				
				if(!empty($ticket_updated) && !is_numeric($ticket_updated)) {
				
					$ticket_updated = cqpim_convert_date($ticket_updated, 'd/m/Y H:i');
				
				}	
				
				update_post_meta($invoice->ID, 'last_updated', $ticket_updated);

				$ticket_updates = get_post_meta($invoice->ID, 'ticket_updates', true);
				
				if(!empty($ticket_updates)) {
				
					foreach($ticket_updates as $key => $update) {
					
						if(!empty($update['time']) && !is_numeric($update['time'])) {
						
							$ticket_updates[$key]['time'] = cqpim_convert_date($update['time'], 'd/m/Y H:i');
						
						}						
					
					}
				
				}
				
				update_post_meta($invoice->ID, 'ticket_updates', $ticket_updates);

			}
			
			update_option('twopoint7_completed', '1');
			
		}
	
	}
	
	add_action('admin_init', 'cqpim_twopointsevenfive_compat');
	
	function cqpim_twopointsevenfive_compat() {
	
		$complete = get_option('twopoint7five_completed');
		
		if($complete != '1') {
		
			update_option('enable_quotes', 1);
			
			update_option('enable_project_creation', 1);
			
			update_option('enable_project_contracts', 1);
			
			update_option('auto_contract', 1);
			
			update_option('invoice_workflow', 0);
			
			update_option('auto_send_invoices', 1);		
		
			update_option('twopoint7five_completed', '1');
		
		}
		
	}
	
	add_action('admin_init', 'cqpim_twopointeight_compat');
	
	function cqpim_twopointeight_compat() {
	
		$complete = get_option('twopoint8_completed');
		
		if($complete != '1') {
		
			$system_tax_rate = get_option('sales_tax_rate');
			
			$args = array(
			
				'post_type' => array('cqpim_quote', 'cqpim_project', 'cqpim_invoice'),
				
				'posts_per_page' => -1,
				
				'post_status' => array('publish', 'draft', 'private')
			
			);
			
			$posts = get_posts($args);
			
			foreach($posts as $post) {

				if(empty($system_tax_rate)) {
				
					update_post_meta($post->ID, 'tax_applicable', 0);
					update_post_meta($post->ID, 'tax_set', 1);
					update_post_meta($post->ID, 'tax_rate', 0);
				
				} else { 
				
					update_post_meta($post->ID, 'tax_applicable', 1);
					update_post_meta($post->ID, 'tax_set', 0);
					update_post_meta($post->ID, 'tax_rate', $system_tax_rate);
				
				}
			
			}
		
			update_option('twopoint8_completed', '1');
		
		}
		
	}
	
	add_action('admin_init', 'cqpim_twopointnine_compat');
	
	function cqpim_twopointnine_compat() {
	
		$complete = get_option('twopoint9_completed');
		
		if($complete != '1') {
		
			update_option('cqpim_allowed_extensions', 'png,jpg,gif,zip,doc,docx,xls,xlsx,pdf');
		
			update_option('twopoint9_completed', '1');
		
		}
		
	}

	
	
	
	
	
	